
/**
 * Drupal initializer.
 * Launch as behavior and pull variables from config.
 */


// Prevent multiple inits in modules that re-trigger the document context.
var linkPurposeOnce;

Drupal.behaviors.linkPurpose = {
    attach: function (context, settings) {

        if (context === document && !linkPurposeOnce && CSS.supports("selector(:is(body))")) {
            // "Supports" selector drops old browsers.
            linkPurposeOnce = true;

            const invalidClass = /([^A-Za-z0-9\-\_])+/g;
            const invalidClasses = /([^A-Za-z0-9,\-\_])+/g;

            const sanitizeClass = function(string, defaultString) {
                if (string) {
                    return string.replaceAll(invalidClass, '')
                } else {
                    return defaultString;
                }
            }

            const classArray = function(source, defaults) {

                if (!source) {
                    return defaults;
                }

                // normalize
                source = source.replaceAll(', ', ',');
                source = source.replaceAll(' ', ',');

                // sanitize
                source = source.replaceAll(invalidClasses, '');

                return source.split(',');
            }
            
            let ignore = drupalSettings.linkpurpose.ignore ? `:not(${drupalSettings.linkpurpose.ignore})` : '';

            let options = {
              
                domain: drupalSettings.linkpurpose.domain,
                roots: drupalSettings.linkpurpose.roots ? drupalSettings.linkpurpose.roots : 'body',
                shadowComponents: drupalSettings.linkpurpose.shadowComponents ? drupalSettings.linkpurpose.shadowComponents : '',
                ignore: ignore,
                hideIcon: drupalSettings.linkpurpose.hideIcon ? drupalSettings.linkpurpose.hideIcon : false,
                noRunIfPresent: drupalSettings.linkpurpose.noRunIfPresent ? drupalSettings.linkpurpose.noRunIfPresent : '#quickedit-entity-toolbar, .layout-builder-form',
                noRunIfAbsent: drupalSettings.linkpurpose.noRunIfAbsent ? drupalSettings.linkpurpose.noRunIfAbsent : '',
                baseSelector: drupalSettings.linkpurpose.baseSelector ? drupalSettings.linkpurpose.baseSelector : 'a[href]',
                baseLinkClass: sanitizeClass(drupalSettings.linkpurpose.baseLinkClass, 'link-purpose'),
                baseIconWrapperClass: sanitizeClass(drupalSettings.linkpurpose.baseIconWrapperClass, 'link-purpose-icon'),
                noBreakClass: sanitizeClass(drupalSettings.linkpurpose.noBreakClass, 'link-purpose-nobreak'),
                noReferrer: !!drupalSettings.linkpurpose.purposeExternalNoReferrer,

                purposes: {
                    external: {
                        additionalSelector: drupalSettings.linkpurpose.purposeExternalSelector ? drupalSettings.linkpurpose.purposeExternalSelector : '',
                        newWindow: !!drupalSettings.linkpurpose.purposeExternalNewWindow,
                        message: drupalSettings.linkpurpose.purposeExternalMessage ? drupalSettings.linkpurpose.purposeExternalMessage : '(Link is external)',
                        linkClass: sanitizeClass(drupalSettings.linkpurpose.purposeExternalClass, 'link-purpose-external'),
                        iconWrapperClass: sanitizeClass(drupalSettings.linkpurpose.purposeExternalIconWrapperClass, 'link-purpose-external-icon'),
                        iconType: drupalSettings.linkpurpose.purposeExternalIconType ? drupalSettings.linkpurpose.purposeExternalIconType : 'html', 
                        iconPosition: drupalSettings.linkpurpose.purposeExternalIconPosition ? drupalSettings.linkpurpose.purposeExternalIconPosition : 'beforeend', 
                        iconClasses: classArray(drupalSettings.linkpurpose.purposeExternalIconClasses, ['fa-solid', 'fa-up-right-from-square']),
                    },

                    document: {
                        selector: drupalSettings.linkpurpose.purposeDocumentSelector ? drupalSettings.linkpurpose.purposeDocumentSelector : '[href$=\'.pdf\'], [href*=\'.pdf?\'], [href$=\'.doc\'], [href$=\'.docx\'], [href*=\'.doc?\'], [href*=\'.docx?\'], [href$=\'.ppt\'], [href$=\'.pptx\'], [href*=\'.ppt?\'], [href*=\'.pptx?\'], [href^=\'https://docs.google\']',
                        message: drupalSettings.linkpurpose.purposeDocumentMessage ? drupalSettings.linkpurpose.purposeDocumentMessage : '(Link opens document)',
                        linkClass: sanitizeClass(drupalSettings.linkpurpose.purposeDocumentClass, 'link-purpose-document'),
                        iconWrapperClass: sanitizeClass(drupalSettings.linkpurpose.purposeDocumentIconWrapperClass, 'link-purpose-document-icon'),
                        iconType: drupalSettings.linkpurpose.purposeDocumentIconType ? drupalSettings.linkpurpose.purposeDocumentIconType : 'html',
                        iconPosition: drupalSettings.linkpurpose.purposeDocumentIconPosition ? drupalSettings.linkpurpose.purposeDocumentIconPosition : 'beforeend', 
                        iconClasses: classArray(drupalSettings.linkpurpose.purposeDocumentIconClasses, ['fa-regular', 'fa-file-lines']),
                     },

                    mail: {
                        message: drupalSettings.linkpurpose.purposeMailMessage ? drupalSettings.linkpurpose.purposeMailMessage : '(Link sends Email)',
                        linkClass: sanitizeClass(drupalSettings.linkpurpose.purposeMailClass, 'link-purpose-mailto'),
                        iconWrapperClass: sanitizeClass(drupalSettings.linkpurpose.purposeMailIconWrapperClass, 'link-purpose-mail-icon'),
                        iconType: drupalSettings.linkpurpose.purposeMailIconType ? drupalSettings.linkpurpose.purposeMailIconType : 'html',
                        iconPosition: drupalSettings.linkpurpose.purposeMailIconPosition ? drupalSettings.linkpurpose.purposeMailIconPosition : 'beforeend', 
                        iconClasses: classArray(drupalSettings.linkpurpose.purposeMailIconClasses,['fa-regular', 'fa-envelope']),
                    },

                    newWindow: {
                        message: drupalSettings.linkpurpose.purposeNewWindowMessage ? drupalSettings.linkpurpose.purposeNewWindowMessage : '(Link is telephone number)',
                        linkClass: sanitizeClass(drupalSettings.linkpurpose.purposeNewWindowClass, 'link-purpose-window'),
                        iconWrapperClass: sanitizeClass(drupalSettings.linkpurpose.purposeNewWindowIconWrapperClass, 'link-purpose-window-icon'),
                        iconType: drupalSettings.linkpurpose.purposeNewWindowIconType ? drupalSettings.linkpurpose.purposeNewWindowIconType : 'html', // html, src or classes
                        iconPosition: drupalSettings.linkpurpose.purposeNewWindowIconPosition ? drupalSettings.linkpurpose.purposeNewWindowIconPosition : 'beforeend', 
                        iconClasses: classArray(drupalSettings.linkpurpose.purposeNewWindowIconClasses, ['fa-regular', 'fa-window-restore']),
                    },

                    tel: {
                        message: drupalSettings.linkpurpose.purposeTelMessage ? drupalSettings.linkpurpose.purposeTelMessage : '(New window)',
                        linkClass: sanitizeClass(drupalSettings.linkpurpose.purposeTelClass, 'link-purpose-window'),
                        iconWrapperClass: sanitizeClass(drupalSettings.linkpurpose.purposeTelIconWrapperClass, 'link-purpose-window-icon'),
                        iconType: drupalSettings.linkpurpose.purposeTelIconType ? drupalSettings.linkpurpose.purposeTelIconType : 'html', // html, src or classes
                        iconPosition: drupalSettings.linkpurpose.purposeTelIconPosition ? drupalSettings.linkpurpose.purposeTelIconPosition : 'beforeend', 
                        iconClasses: classArray(drupalSettings.linkpurpose.purposeTelIconClasses, ['fa-solid', 'fa-square-phone']),
                    },
                }
            }

            const linkPurpose = new LinkPurpose(options);

        }
    }
};
